# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKSTATS_METADATA = Metadata(
    id="1c376e03915b6fa825105cf6a3a1f00bd7cafd03.boutiques",
    name="tckstats",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TckstatsOutputParamsDictNoTag = typing.TypedDict('_TckstatsOutputParamsDictNoTag', {
    "field": str,
})
TckstatsOutputParamsDictTagged = typing.TypedDict('TckstatsOutputParamsDictTagged', {
    "@type": typing.Literal["output"],
    "field": str,
})
TckstatsOutputParamsDict = _TckstatsOutputParamsDictNoTag | TckstatsOutputParamsDictTagged


_TckstatsConfigParamsDictNoTag = typing.TypedDict('_TckstatsConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TckstatsConfigParamsDictTagged = typing.TypedDict('TckstatsConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TckstatsConfigParamsDict = _TckstatsConfigParamsDictNoTag | TckstatsConfigParamsDictTagged


_TckstatsParamsDictNoTag = typing.TypedDict('_TckstatsParamsDictNoTag', {
    "output": typing.NotRequired[list[TckstatsOutputParamsDict] | None],
    "histogram": typing.NotRequired[str | None],
    "dump": typing.NotRequired[str | None],
    "ignorezero": bool,
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckstatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks_in": InputPathType,
})
TckstatsParamsDictTagged = typing.TypedDict('TckstatsParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tckstats"],
    "output": typing.NotRequired[list[TckstatsOutputParamsDict] | None],
    "histogram": typing.NotRequired[str | None],
    "dump": typing.NotRequired[str | None],
    "ignorezero": bool,
    "tck_weights_in": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckstatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks_in": InputPathType,
})
TckstatsParamsDict = _TckstatsParamsDictNoTag | TckstatsParamsDictTagged


def tckstats_output(
    field: str,
) -> TckstatsOutputParamsDictTagged:
    """
    Build parameters.
    
    Args:
        field: output only the field specified. Multiple such options can be\
            supplied if required. Choices are: mean, median, std, min, max, count.\
            Useful for use in scripts.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "output",
        "field": field,
    }
    return params


def tckstats_output_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckstatsOutputParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("field", None) is None:
        raise StyxValidationError("`field` must not be None")
    if not isinstance(params["field"], str):
        raise StyxValidationError(f'`field` has the wrong type: Received `{type(params.get("field", None))}` expected `str`')


def tckstats_output_cargs(
    params: TckstatsOutputParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-output")
    cargs.append(params.get("field", None))
    return cargs


def tckstats_config(
    key: str,
    value: str,
) -> TckstatsConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tckstats_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckstatsConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tckstats_config_cargs(
    params: TckstatsConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class TckstatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TckstatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    histogram: OutputPathType | None
    """output a histogram of streamline lengths """
    dump: OutputPathType | None
    """dump the streamlines lengths to a text file """


def tckstats_params(
    tracks_in: InputPathType,
    output: list[TckstatsOutputParamsDict] | None = None,
    histogram: str | None = None,
    dump: str | None = None,
    ignorezero: bool = False,
    tck_weights_in: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckstatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TckstatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tracks_in: the input track file.
        output: output only the field specified. Multiple such options can be\
            supplied if required. Choices are: mean, median, std, min, max, count.\
            Useful for use in scripts.
        histogram: output a histogram of streamline lengths.
        dump: dump the streamlines lengths to a text file.
        ignorezero: do not generate a warning if the track file contains\
            streamlines with zero length.
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tckstats",
        "ignorezero": ignorezero,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tracks_in": tracks_in,
    }
    if output is not None:
        params["output"] = output
    if histogram is not None:
        params["histogram"] = histogram
    if dump is not None:
        params["dump"] = dump
    if tck_weights_in is not None:
        params["tck_weights_in"] = tck_weights_in
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tckstats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckstatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("output", None) is not None:
        if not isinstance(params["output"], list):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `list[TckstatsOutputParamsDict] | None`')
        for e in params["output"]:
            tckstats_output_validate(e)
    if params.get("histogram", None) is not None:
        if not isinstance(params["histogram"], str):
            raise StyxValidationError(f'`histogram` has the wrong type: Received `{type(params.get("histogram", None))}` expected `str | None`')
    if params.get("dump", None) is not None:
        if not isinstance(params["dump"], str):
            raise StyxValidationError(f'`dump` has the wrong type: Received `{type(params.get("dump", None))}` expected `str | None`')
    if params.get("ignorezero", False) is None:
        raise StyxValidationError("`ignorezero` must not be None")
    if not isinstance(params["ignorezero"], bool):
        raise StyxValidationError(f'`ignorezero` has the wrong type: Received `{type(params.get("ignorezero", False))}` expected `bool`')
    if params.get("tck_weights_in", None) is not None:
        if not isinstance(params["tck_weights_in"], (pathlib.Path, str)):
            raise StyxValidationError(f'`tck_weights_in` has the wrong type: Received `{type(params.get("tck_weights_in", None))}` expected `InputPathType | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TckstatsConfigParamsDict] | None`')
        for e in params["config"]:
            tckstats_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tracks_in", None) is None:
        raise StyxValidationError("`tracks_in` must not be None")
    if not isinstance(params["tracks_in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks_in` has the wrong type: Received `{type(params.get("tracks_in", None))}` expected `InputPathType`')


def tckstats_cargs(
    params: TckstatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tckstats")
    if params.get("output", None) is not None:
        cargs.extend([a for c in [tckstats_output_cargs(s, execution) for s in params.get("output", None)] for a in c])
    if params.get("histogram", None) is not None:
        cargs.extend([
            "-histogram",
            params.get("histogram", None)
        ])
    if params.get("dump", None) is not None:
        cargs.extend([
            "-dump",
            params.get("dump", None)
        ])
    if params.get("ignorezero", False):
        cargs.append("-ignorezero")
    if params.get("tck_weights_in", None) is not None:
        cargs.extend([
            "-tck_weights_in",
            execution.input_file(params.get("tck_weights_in", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tckstats_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tracks_in", None)))
    return cargs


def tckstats_outputs(
    params: TckstatsParamsDict,
    execution: Execution,
) -> TckstatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TckstatsOutputs(
        root=execution.output_file("."),
        histogram=execution.output_file(params.get("histogram", None)) if (params.get("histogram") is not None) else None,
        dump=execution.output_file(params.get("dump", None)) if (params.get("dump") is not None) else None,
    )
    return ret


def tckstats_execute(
    params: TckstatsParamsDict,
    runner: Runner | None = None,
) -> TckstatsOutputs:
    """
    tckstats
    
    Calculate statistics on streamlines lengths.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckstatsOutputs`).
    """
    tckstats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKSTATS_METADATA)
    params = execution.params(params)
    cargs = tckstats_cargs(params, execution)
    ret = tckstats_outputs(params, execution)
    execution.run(cargs)
    return ret


def tckstats(
    tracks_in: InputPathType,
    output: list[TckstatsOutputParamsDict] | None = None,
    histogram: str | None = None,
    dump: str | None = None,
    ignorezero: bool = False,
    tck_weights_in: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckstatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TckstatsOutputs:
    """
    tckstats
    
    Calculate statistics on streamlines lengths.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks_in: the input track file.
        output: output only the field specified. Multiple such options can be\
            supplied if required. Choices are: mean, median, std, min, max, count.\
            Useful for use in scripts.
        histogram: output a histogram of streamline lengths.
        dump: dump the streamlines lengths to a text file.
        ignorezero: do not generate a warning if the track file contains\
            streamlines with zero length.
        tck_weights_in: specify a text scalar file containing the streamline\
            weights.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckstatsOutputs`).
    """
    params = tckstats_params(
        output=output,
        histogram=histogram,
        dump=dump,
        ignorezero=ignorezero,
        tck_weights_in=tck_weights_in,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tracks_in=tracks_in,
    )
    return tckstats_execute(params, runner)


__all__ = [
    "TCKSTATS_METADATA",
    "TckstatsConfigParamsDict",
    "TckstatsConfigParamsDictTagged",
    "TckstatsOutputParamsDict",
    "TckstatsOutputParamsDictTagged",
    "TckstatsOutputs",
    "TckstatsParamsDict",
    "TckstatsParamsDictTagged",
    "tckstats",
    "tckstats_config",
    "tckstats_execute",
    "tckstats_output",
    "tckstats_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MTNORMALISE_METADATA = Metadata(
    id="3bc2bbe8af34500259b7245db2d18482e852bfea.boutiques",
    name="mtnormalise",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MtnormaliseConfigParamsDictNoTag = typing.TypedDict('_MtnormaliseConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MtnormaliseConfigParamsDictTagged = typing.TypedDict('MtnormaliseConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MtnormaliseConfigParamsDict = _MtnormaliseConfigParamsDictNoTag | MtnormaliseConfigParamsDictTagged


_MtnormaliseInputOutputParamsDictNoTag = typing.TypedDict('_MtnormaliseInputOutputParamsDictNoTag', {
    "input": InputPathType,
    "output": str,
})
MtnormaliseInputOutputParamsDictTagged = typing.TypedDict('MtnormaliseInputOutputParamsDictTagged', {
    "@type": typing.Literal["input_output"],
    "input": InputPathType,
    "output": str,
})
MtnormaliseInputOutputParamsDict = _MtnormaliseInputOutputParamsDictNoTag | MtnormaliseInputOutputParamsDictTagged


_MtnormaliseParamsDictNoTag = typing.TypedDict('_MtnormaliseParamsDictNoTag', {
    "mask": InputPathType,
    "order": typing.NotRequired[str | None],
    "niter": typing.NotRequired[list[int] | None],
    "reference": typing.NotRequired[float | None],
    "balanced": bool,
    "check_norm": typing.NotRequired[str | None],
    "check_mask": typing.NotRequired[str | None],
    "check_factors": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MtnormaliseConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input_output": list[MtnormaliseInputOutputParamsDict],
})
MtnormaliseParamsDictTagged = typing.TypedDict('MtnormaliseParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mtnormalise"],
    "mask": InputPathType,
    "order": typing.NotRequired[str | None],
    "niter": typing.NotRequired[list[int] | None],
    "reference": typing.NotRequired[float | None],
    "balanced": bool,
    "check_norm": typing.NotRequired[str | None],
    "check_mask": typing.NotRequired[str | None],
    "check_factors": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MtnormaliseConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input_output": list[MtnormaliseInputOutputParamsDict],
})
MtnormaliseParamsDict = _MtnormaliseParamsDictNoTag | MtnormaliseParamsDictTagged


def mtnormalise_config(
    key: str,
    value: str,
) -> MtnormaliseConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mtnormalise_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MtnormaliseConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mtnormalise_config_cargs(
    params: MtnormaliseConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MtnormaliseInputOutputOutputs(typing.NamedTuple):
    """
    Output object returned when calling `list[MtnormaliseInputOutputParamsDict](...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """output normalised tissue compartment image."""


def mtnormalise_input_output(
    input_: InputPathType,
    output: str,
) -> MtnormaliseInputOutputParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: input tissue compartment image.
        output: output normalised tissue compartment image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "input_output",
        "input": input_,
        "output": output,
    }
    return params


def mtnormalise_input_output_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MtnormaliseInputOutputParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mtnormalise_input_output_cargs(
    params: MtnormaliseInputOutputParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def mtnormalise_input_output_outputs(
    params: MtnormaliseInputOutputParamsDict,
    execution: Execution,
) -> MtnormaliseInputOutputOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MtnormaliseInputOutputOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


class MtnormaliseOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MtnormaliseParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    check_norm: OutputPathType | None
    """output the final estimated spatially varying intensity level that is used
    for normalisation. """
    check_mask: OutputPathType | None
    """output the final mask used to compute the normalisation. This mask
    excludes regions identified as outliers by the optimisation process. """
    check_factors: OutputPathType | None
    """output the tissue balance factors computed during normalisation. """
    input_output: list[MtnormaliseInputOutputOutputs]
    """Outputs from `mtnormalise_input_output_outputs`.This is a list of outputs
    with the same length and order as the inputs."""


def mtnormalise_params(
    mask: InputPathType,
    input_output: list[MtnormaliseInputOutputParamsDict],
    order: str | None = None,
    niter: list[int] | None = None,
    reference: float | None = None,
    balanced: bool = False,
    check_norm: str | None = None,
    check_mask: str | None = None,
    check_factors: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MtnormaliseConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MtnormaliseParamsDictTagged:
    """
    Build parameters.
    
    Args:
        mask: the mask defines the data used to compute the intensity\
            normalisation. This option is mandatory.
        input_output: list of all input and output tissue compartment files\
            (see example usage).
        order: the maximum order of the polynomial basis used to fit the\
            normalisation field in the log-domain. An order of 0 is equivalent to\
            not allowing spatial variance of the intensity normalisation factor.\
            (default: 3).
        niter: set the number of iterations. The first (and potentially only)\
            entry applies to the main loop. If supplied as a comma-separated list\
            of integers, the second entry applies to the inner loop to update the\
            balance factors (default: 15,7).
        reference: specify the (positive) reference value to which the summed\
            tissue compartments will be normalised. (default: 0.282095, SH DC term\
            for unit angular integral).
        balanced: incorporate the per-tissue balancing factors into scaling of\
            the output images (NOTE: use of this option has critical consequences\
            for AFD intensity normalisation; should not be used unless these\
            consequences are fully understood).
        check_norm: output the final estimated spatially varying intensity\
            level that is used for normalisation.
        check_mask: output the final mask used to compute the normalisation.\
            This mask excludes regions identified as outliers by the optimisation\
            process.
        check_factors: output the tissue balance factors computed during\
            normalisation.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mtnormalise",
        "mask": mask,
        "balanced": balanced,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input_output": input_output,
    }
    if order is not None:
        params["order"] = order
    if niter is not None:
        params["niter"] = niter
    if reference is not None:
        params["reference"] = reference
    if check_norm is not None:
        params["check_norm"] = check_norm
    if check_mask is not None:
        params["check_mask"] = check_mask
    if check_factors is not None:
        params["check_factors"] = check_factors
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mtnormalise_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MtnormaliseParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mask", None) is None:
        raise StyxValidationError("`mask` must not be None")
    if not isinstance(params["mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType`')
    if params.get("order", None) is not None:
        if not isinstance(params["order"], str):
            raise StyxValidationError(f'`order` has the wrong type: Received `{type(params.get("order", None))}` expected `str | None`')
    if params.get("niter", None) is not None:
        if not isinstance(params["niter"], list):
            raise StyxValidationError(f'`niter` has the wrong type: Received `{type(params.get("niter", None))}` expected `list[int] | None`')
        for e in params["niter"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`niter` has the wrong type: Received `{type(params.get("niter", None))}` expected `list[int] | None`')
    if params.get("reference", None) is not None:
        if not isinstance(params["reference"], (float, int)):
            raise StyxValidationError(f'`reference` has the wrong type: Received `{type(params.get("reference", None))}` expected `float | None`')
    if params.get("balanced", False) is None:
        raise StyxValidationError("`balanced` must not be None")
    if not isinstance(params["balanced"], bool):
        raise StyxValidationError(f'`balanced` has the wrong type: Received `{type(params.get("balanced", False))}` expected `bool`')
    if params.get("check_norm", None) is not None:
        if not isinstance(params["check_norm"], str):
            raise StyxValidationError(f'`check_norm` has the wrong type: Received `{type(params.get("check_norm", None))}` expected `str | None`')
    if params.get("check_mask", None) is not None:
        if not isinstance(params["check_mask"], str):
            raise StyxValidationError(f'`check_mask` has the wrong type: Received `{type(params.get("check_mask", None))}` expected `str | None`')
    if params.get("check_factors", None) is not None:
        if not isinstance(params["check_factors"], str):
            raise StyxValidationError(f'`check_factors` has the wrong type: Received `{type(params.get("check_factors", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MtnormaliseConfigParamsDict] | None`')
        for e in params["config"]:
            mtnormalise_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input_output", None) is None:
        raise StyxValidationError("`input_output` must not be None")
    if not isinstance(params["input_output"], list):
        raise StyxValidationError(f'`input_output` has the wrong type: Received `{type(params.get("input_output", None))}` expected `list[MtnormaliseInputOutputParamsDict]`')
    for e in params["input_output"]:
        mtnormalise_input_output_validate(e)


def mtnormalise_cargs(
    params: MtnormaliseParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mtnormalise")
    cargs.extend([
        "-mask",
        execution.input_file(params.get("mask", None))
    ])
    if params.get("order", None) is not None:
        cargs.extend([
            "-order",
            params.get("order", None)
        ])
    if params.get("niter", None) is not None:
        cargs.extend([
            "-niter",
            ",".join(map(str, params.get("niter", None)))
        ])
    if params.get("reference", None) is not None:
        cargs.extend([
            "-reference",
            str(params.get("reference", None))
        ])
    if params.get("balanced", False):
        cargs.append("-balanced")
    if params.get("check_norm", None) is not None:
        cargs.extend([
            "-check_norm",
            params.get("check_norm", None)
        ])
    if params.get("check_mask", None) is not None:
        cargs.extend([
            "-check_mask",
            params.get("check_mask", None)
        ])
    if params.get("check_factors", None) is not None:
        cargs.extend([
            "-check_factors",
            params.get("check_factors", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mtnormalise_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.extend([a for c in [mtnormalise_input_output_cargs(s, execution) for s in params.get("input_output", None)] for a in c])
    return cargs


def mtnormalise_outputs(
    params: MtnormaliseParamsDict,
    execution: Execution,
) -> MtnormaliseOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MtnormaliseOutputs(
        root=execution.output_file("."),
        check_norm=execution.output_file(params.get("check_norm", None)) if (params.get("check_norm") is not None) else None,
        check_mask=execution.output_file(params.get("check_mask", None)) if (params.get("check_mask") is not None) else None,
        check_factors=execution.output_file(params.get("check_factors", None)) if (params.get("check_factors") is not None) else None,
        input_output=[mtnormalise_input_output_outputs(i, execution) if mtnormalise_input_output_outputs else None for i in params.get("input_output")],
    )
    return ret


def mtnormalise_execute(
    params: MtnormaliseParamsDict,
    runner: Runner | None = None,
) -> MtnormaliseOutputs:
    """
    mtnormalise
    
    Multi-tissue informed log-domain intensity normalisation.
    
    This command takes as input any number of tissue components (e.g. from
    multi-tissue CSD) and outputs corresponding normalised tissue components
    corrected for the effects of (residual) intensity inhomogeneities. Intensity
    normalisation is performed by optimising the voxel-wise sum of all tissue
    compartments towards a constant value, under constraints of spatial
    smoothness (polynomial basis of a given order). Different to the Raffelt et
    al. 2017 abstract, this algorithm performs this task in the log-domain
    instead, with added gradual outlier rejection, different handling of the
    balancing factors between tissue compartments and a different iteration
    structure.
    
    The -mask option is mandatory and is optimally provided with a brain mask
    (such as the one obtained from dwi2mask earlier in the processing pipeline).
    Outlier areas with exceptionally low or high combined tissue contributions
    are accounted for and reoptimised as the intensity inhomogeneity estimation
    becomes more accurate.
    
    References:
    
    Raffelt, D.; Dhollander, T.; Tournier, J.-D.; Tabbara, R.; Smith, R. E.;
    Pierre, E. & Connelly, A. Bias Field Correction and Intensity Normalisation
    for Quantitative Analysis of Apparent Fibre Density. In Proc. ISMRM, 2017,
    26, 3541
    
    Dhollander, T.; Tabbara, R.; Rosnarho-Tornstrand, J.; Tournier, J.-D.;
    Raffelt, D. & Connelly, A. Multi-tissue log-domain intensity and
    inhomogeneity normalisation for quantitative apparent fibre density. In
    Proc. ISMRM, 2021, 29, 2472.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MtnormaliseOutputs`).
    """
    mtnormalise_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MTNORMALISE_METADATA)
    params = execution.params(params)
    cargs = mtnormalise_cargs(params, execution)
    ret = mtnormalise_outputs(params, execution)
    execution.run(cargs)
    return ret


def mtnormalise(
    mask: InputPathType,
    input_output: list[MtnormaliseInputOutputParamsDict],
    order: str | None = None,
    niter: list[int] | None = None,
    reference: float | None = None,
    balanced: bool = False,
    check_norm: str | None = None,
    check_mask: str | None = None,
    check_factors: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MtnormaliseConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MtnormaliseOutputs:
    """
    mtnormalise
    
    Multi-tissue informed log-domain intensity normalisation.
    
    This command takes as input any number of tissue components (e.g. from
    multi-tissue CSD) and outputs corresponding normalised tissue components
    corrected for the effects of (residual) intensity inhomogeneities. Intensity
    normalisation is performed by optimising the voxel-wise sum of all tissue
    compartments towards a constant value, under constraints of spatial
    smoothness (polynomial basis of a given order). Different to the Raffelt et
    al. 2017 abstract, this algorithm performs this task in the log-domain
    instead, with added gradual outlier rejection, different handling of the
    balancing factors between tissue compartments and a different iteration
    structure.
    
    The -mask option is mandatory and is optimally provided with a brain mask
    (such as the one obtained from dwi2mask earlier in the processing pipeline).
    Outlier areas with exceptionally low or high combined tissue contributions
    are accounted for and reoptimised as the intensity inhomogeneity estimation
    becomes more accurate.
    
    References:
    
    Raffelt, D.; Dhollander, T.; Tournier, J.-D.; Tabbara, R.; Smith, R. E.;
    Pierre, E. & Connelly, A. Bias Field Correction and Intensity Normalisation
    for Quantitative Analysis of Apparent Fibre Density. In Proc. ISMRM, 2017,
    26, 3541
    
    Dhollander, T.; Tabbara, R.; Rosnarho-Tornstrand, J.; Tournier, J.-D.;
    Raffelt, D. & Connelly, A. Multi-tissue log-domain intensity and
    inhomogeneity normalisation for quantitative apparent fibre density. In
    Proc. ISMRM, 2021, 29, 2472.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        mask: the mask defines the data used to compute the intensity\
            normalisation. This option is mandatory.
        input_output: list of all input and output tissue compartment files\
            (see example usage).
        order: the maximum order of the polynomial basis used to fit the\
            normalisation field in the log-domain. An order of 0 is equivalent to\
            not allowing spatial variance of the intensity normalisation factor.\
            (default: 3).
        niter: set the number of iterations. The first (and potentially only)\
            entry applies to the main loop. If supplied as a comma-separated list\
            of integers, the second entry applies to the inner loop to update the\
            balance factors (default: 15,7).
        reference: specify the (positive) reference value to which the summed\
            tissue compartments will be normalised. (default: 0.282095, SH DC term\
            for unit angular integral).
        balanced: incorporate the per-tissue balancing factors into scaling of\
            the output images (NOTE: use of this option has critical consequences\
            for AFD intensity normalisation; should not be used unless these\
            consequences are fully understood).
        check_norm: output the final estimated spatially varying intensity\
            level that is used for normalisation.
        check_mask: output the final mask used to compute the normalisation.\
            This mask excludes regions identified as outliers by the optimisation\
            process.
        check_factors: output the tissue balance factors computed during\
            normalisation.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MtnormaliseOutputs`).
    """
    params = mtnormalise_params(
        mask=mask,
        order=order,
        niter=niter,
        reference=reference,
        balanced=balanced,
        check_norm=check_norm,
        check_mask=check_mask,
        check_factors=check_factors,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_output=input_output,
    )
    return mtnormalise_execute(params, runner)


__all__ = [
    "MTNORMALISE_METADATA",
    "MtnormaliseConfigParamsDict",
    "MtnormaliseConfigParamsDictTagged",
    "MtnormaliseInputOutputOutputs",
    "MtnormaliseInputOutputParamsDict",
    "MtnormaliseInputOutputParamsDictTagged",
    "MtnormaliseOutputs",
    "MtnormaliseParamsDict",
    "MtnormaliseParamsDictTagged",
    "mtnormalise",
    "mtnormalise_config",
    "mtnormalise_execute",
    "mtnormalise_input_output",
    "mtnormalise_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKTRANSFORM_METADATA = Metadata(
    id="e269e831330358a696661d691299c44a7c3630bb.boutiques",
    name="tcktransform",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TcktransformConfigParamsDictNoTag = typing.TypedDict('_TcktransformConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TcktransformConfigParamsDictTagged = typing.TypedDict('TcktransformConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TcktransformConfigParamsDict = _TcktransformConfigParamsDictNoTag | TcktransformConfigParamsDictTagged


_TcktransformParamsDictNoTag = typing.TypedDict('_TcktransformParamsDictNoTag', {
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TcktransformConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "transform": InputPathType,
    "output": str,
})
TcktransformParamsDictTagged = typing.TypedDict('TcktransformParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tcktransform"],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TcktransformConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "transform": InputPathType,
    "output": str,
})
TcktransformParamsDict = _TcktransformParamsDictNoTag | TcktransformParamsDictTagged


def tcktransform_config(
    key: str,
    value: str,
) -> TcktransformConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tcktransform_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TcktransformConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tcktransform_config_cargs(
    params: TcktransformConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class TcktransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TcktransformParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output track file"""


def tcktransform_params(
    tracks: InputPathType,
    transform: InputPathType,
    output: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TcktransformConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TcktransformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tracks: the input track file.
        transform: the image containing the transform.
        output: the output track file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tcktransform",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tracks": tracks,
        "transform": transform,
        "output": output,
    }
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tcktransform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TcktransformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TcktransformConfigParamsDict] | None`')
        for e in params["config"]:
            tcktransform_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tracks", None) is None:
        raise StyxValidationError("`tracks` must not be None")
    if not isinstance(params["tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks` has the wrong type: Received `{type(params.get("tracks", None))}` expected `InputPathType`')
    if params.get("transform", None) is None:
        raise StyxValidationError("`transform` must not be None")
    if not isinstance(params["transform"], (pathlib.Path, str)):
        raise StyxValidationError(f'`transform` has the wrong type: Received `{type(params.get("transform", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def tcktransform_cargs(
    params: TcktransformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tcktransform")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tcktransform_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tracks", None)))
    cargs.append(execution.input_file(params.get("transform", None)))
    cargs.append(params.get("output", None))
    return cargs


def tcktransform_outputs(
    params: TcktransformParamsDict,
    execution: Execution,
) -> TcktransformOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TcktransformOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def tcktransform_execute(
    params: TcktransformParamsDict,
    runner: Runner | None = None,
) -> TcktransformOutputs:
    """
    tcktransform
    
    Apply a spatial transformation to a tracks file.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TcktransformOutputs`).
    """
    tcktransform_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKTRANSFORM_METADATA)
    params = execution.params(params)
    cargs = tcktransform_cargs(params, execution)
    ret = tcktransform_outputs(params, execution)
    execution.run(cargs)
    return ret


def tcktransform(
    tracks: InputPathType,
    transform: InputPathType,
    output: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TcktransformConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TcktransformOutputs:
    """
    tcktransform
    
    Apply a spatial transformation to a tracks file.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks: the input track file.
        transform: the image containing the transform.
        output: the output track file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TcktransformOutputs`).
    """
    params = tcktransform_params(
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tracks=tracks,
        transform=transform,
        output=output,
    )
    return tcktransform_execute(params, runner)


__all__ = [
    "TCKTRANSFORM_METADATA",
    "TcktransformConfigParamsDict",
    "TcktransformConfigParamsDictTagged",
    "TcktransformOutputs",
    "TcktransformParamsDict",
    "TcktransformParamsDictTagged",
    "tcktransform",
    "tcktransform_config",
    "tcktransform_execute",
    "tcktransform_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

AMP2RESPONSE_METADATA = Metadata(
    id="f0088602c41a6ff20ec7c430ad21aee73ff1f3e4.boutiques",
    name="amp2response",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Amp2responseConfigParamsDictNoTag = typing.TypedDict('_Amp2responseConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Amp2responseConfigParamsDictTagged = typing.TypedDict('Amp2responseConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Amp2responseConfigParamsDict = _Amp2responseConfigParamsDictNoTag | Amp2responseConfigParamsDictTagged


_Amp2responseParamsDictNoTag = typing.TypedDict('_Amp2responseParamsDictNoTag', {
    "isotropic": bool,
    "noconstraint": bool,
    "directions": typing.NotRequired[InputPathType | None],
    "shells": typing.NotRequired[list[float] | None],
    "lmax": typing.NotRequired[list[int] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Amp2responseConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "amps": InputPathType,
    "mask": InputPathType,
    "directions_1": InputPathType,
    "response": str,
})
Amp2responseParamsDictTagged = typing.TypedDict('Amp2responseParamsDictTagged', {
    "@type": typing.Literal["mrtrix/amp2response"],
    "isotropic": bool,
    "noconstraint": bool,
    "directions": typing.NotRequired[InputPathType | None],
    "shells": typing.NotRequired[list[float] | None],
    "lmax": typing.NotRequired[list[int] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Amp2responseConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "amps": InputPathType,
    "mask": InputPathType,
    "directions_1": InputPathType,
    "response": str,
})
Amp2responseParamsDict = _Amp2responseParamsDictNoTag | Amp2responseParamsDictTagged


def amp2response_config(
    key: str,
    value: str,
) -> Amp2responseConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def amp2response_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Amp2responseConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def amp2response_config_cargs(
    params: Amp2responseConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Amp2responseOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Amp2responseParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    response: OutputPathType
    """the output zonal spherical harmonic coefficients"""


def amp2response_params(
    amps: InputPathType,
    mask: InputPathType,
    directions_1: InputPathType,
    response: str,
    isotropic: bool = False,
    noconstraint: bool = False,
    directions: InputPathType | None = None,
    shells: list[float] | None = None,
    lmax: list[int] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Amp2responseConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Amp2responseParamsDictTagged:
    """
    Build parameters.
    
    Args:
        amps: the amplitudes image.
        mask: the mask containing the voxels from which to estimate the\
            response function.
        directions_1: a 4D image containing the estimated fibre directions.
        response: the output zonal spherical harmonic coefficients.
        isotropic: estimate an isotropic response function (lmax=0 for all\
            shells).
        noconstraint: disable the non-negativity and monotonicity constraints.
        directions: provide an external text file containing the directions\
            along which the amplitudes are sampled.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        lmax: specify the maximum harmonic degree of the response function to\
            estimate (can be a comma-separated list for multi-shell data).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/amp2response",
        "isotropic": isotropic,
        "noconstraint": noconstraint,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "amps": amps,
        "mask": mask,
        "directions_1": directions_1,
        "response": response,
    }
    if directions is not None:
        params["directions"] = directions
    if shells is not None:
        params["shells"] = shells
    if lmax is not None:
        params["lmax"] = lmax
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def amp2response_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Amp2responseParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("isotropic", False) is None:
        raise StyxValidationError("`isotropic` must not be None")
    if not isinstance(params["isotropic"], bool):
        raise StyxValidationError(f'`isotropic` has the wrong type: Received `{type(params.get("isotropic", False))}` expected `bool`')
    if params.get("noconstraint", False) is None:
        raise StyxValidationError("`noconstraint` must not be None")
    if not isinstance(params["noconstraint"], bool):
        raise StyxValidationError(f'`noconstraint` has the wrong type: Received `{type(params.get("noconstraint", False))}` expected `bool`')
    if params.get("directions", None) is not None:
        if not isinstance(params["directions"], (pathlib.Path, str)):
            raise StyxValidationError(f'`directions` has the wrong type: Received `{type(params.get("directions", None))}` expected `InputPathType | None`')
    if params.get("shells", None) is not None:
        if not isinstance(params["shells"], list):
            raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
        for e in params["shells"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
    if params.get("lmax", None) is not None:
        if not isinstance(params["lmax"], list):
            raise StyxValidationError(f'`lmax` has the wrong type: Received `{type(params.get("lmax", None))}` expected `list[int] | None`')
        for e in params["lmax"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`lmax` has the wrong type: Received `{type(params.get("lmax", None))}` expected `list[int] | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Amp2responseConfigParamsDict] | None`')
        for e in params["config"]:
            amp2response_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("amps", None) is None:
        raise StyxValidationError("`amps` must not be None")
    if not isinstance(params["amps"], (pathlib.Path, str)):
        raise StyxValidationError(f'`amps` has the wrong type: Received `{type(params.get("amps", None))}` expected `InputPathType`')
    if params.get("mask", None) is None:
        raise StyxValidationError("`mask` must not be None")
    if not isinstance(params["mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType`')
    if params.get("directions_1", None) is None:
        raise StyxValidationError("`directions_1` must not be None")
    if not isinstance(params["directions_1"], (pathlib.Path, str)):
        raise StyxValidationError(f'`directions_1` has the wrong type: Received `{type(params.get("directions_1", None))}` expected `InputPathType`')
    if params.get("response", None) is None:
        raise StyxValidationError("`response` must not be None")
    if not isinstance(params["response"], str):
        raise StyxValidationError(f'`response` has the wrong type: Received `{type(params.get("response", None))}` expected `str`')


def amp2response_cargs(
    params: Amp2responseParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("amp2response")
    if params.get("isotropic", False):
        cargs.append("-isotropic")
    if params.get("noconstraint", False):
        cargs.append("-noconstraint")
    if params.get("directions", None) is not None:
        cargs.extend([
            "-directions",
            execution.input_file(params.get("directions", None))
        ])
    if params.get("shells", None) is not None:
        cargs.extend([
            "-shells",
            ",".join(map(str, params.get("shells", None)))
        ])
    if params.get("lmax", None) is not None:
        cargs.extend([
            "-lmax",
            ",".join(map(str, params.get("lmax", None)))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [amp2response_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("amps", None)))
    cargs.append(execution.input_file(params.get("mask", None)))
    cargs.append(execution.input_file(params.get("directions_1", None)))
    cargs.append(params.get("response", None))
    return cargs


def amp2response_outputs(
    params: Amp2responseParamsDict,
    execution: Execution,
) -> Amp2responseOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Amp2responseOutputs(
        root=execution.output_file("."),
        response=execution.output_file(params.get("response", None)),
    )
    return ret


def amp2response_execute(
    params: Amp2responseParamsDict,
    runner: Runner | None = None,
) -> Amp2responseOutputs:
    """
    amp2response
    
    Estimate response function coefficients based on the DWI signal in
    single-fibre voxels.
    
    This command uses the image data from all selected single-fibre voxels
    concurrently, rather than simply averaging their individual spherical
    harmonic coefficients. It also ensures that the response function is
    non-negative, and monotonic (i.e. its amplitude must increase from the fibre
    direction out to the orthogonal plane).
    
    If multi-shell data are provided, and one or more b-value shells are not
    explicitly requested, the command will generate a response function for
    every b-value shell (including b=0 if present).
    
    References:
    
    Smith, R. E.; Dhollander, T. & Connelly, A. Constrained linear least squares
    estimation of anisotropic response function for spherical deconvolution.
    ISMRM Workshop on Breaking the Barriers of Diffusion MRI, 23.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Amp2responseOutputs`).
    """
    amp2response_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(AMP2RESPONSE_METADATA)
    params = execution.params(params)
    cargs = amp2response_cargs(params, execution)
    ret = amp2response_outputs(params, execution)
    execution.run(cargs)
    return ret


def amp2response(
    amps: InputPathType,
    mask: InputPathType,
    directions_1: InputPathType,
    response: str,
    isotropic: bool = False,
    noconstraint: bool = False,
    directions: InputPathType | None = None,
    shells: list[float] | None = None,
    lmax: list[int] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Amp2responseConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Amp2responseOutputs:
    """
    amp2response
    
    Estimate response function coefficients based on the DWI signal in
    single-fibre voxels.
    
    This command uses the image data from all selected single-fibre voxels
    concurrently, rather than simply averaging their individual spherical
    harmonic coefficients. It also ensures that the response function is
    non-negative, and monotonic (i.e. its amplitude must increase from the fibre
    direction out to the orthogonal plane).
    
    If multi-shell data are provided, and one or more b-value shells are not
    explicitly requested, the command will generate a response function for
    every b-value shell (including b=0 if present).
    
    References:
    
    Smith, R. E.; Dhollander, T. & Connelly, A. Constrained linear least squares
    estimation of anisotropic response function for spherical deconvolution.
    ISMRM Workshop on Breaking the Barriers of Diffusion MRI, 23.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        amps: the amplitudes image.
        mask: the mask containing the voxels from which to estimate the\
            response function.
        directions_1: a 4D image containing the estimated fibre directions.
        response: the output zonal spherical harmonic coefficients.
        isotropic: estimate an isotropic response function (lmax=0 for all\
            shells).
        noconstraint: disable the non-negativity and monotonicity constraints.
        directions: provide an external text file containing the directions\
            along which the amplitudes are sampled.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        lmax: specify the maximum harmonic degree of the response function to\
            estimate (can be a comma-separated list for multi-shell data).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Amp2responseOutputs`).
    """
    params = amp2response_params(
        isotropic=isotropic,
        noconstraint=noconstraint,
        directions=directions,
        shells=shells,
        lmax=lmax,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        amps=amps,
        mask=mask,
        directions_1=directions_1,
        response=response,
    )
    return amp2response_execute(params, runner)


__all__ = [
    "AMP2RESPONSE_METADATA",
    "Amp2responseConfigParamsDict",
    "Amp2responseConfigParamsDictTagged",
    "Amp2responseOutputs",
    "Amp2responseParamsDict",
    "Amp2responseParamsDictTagged",
    "amp2response",
    "amp2response_config",
    "amp2response_execute",
    "amp2response_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRSTATS_METADATA = Metadata(
    id="28fb063299fa07970bfa62d606e13e87fd3bc94e.boutiques",
    name="mrstats",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrstatsOutputParamsDictNoTag = typing.TypedDict('_MrstatsOutputParamsDictNoTag', {
    "field": str,
})
MrstatsOutputParamsDictTagged = typing.TypedDict('MrstatsOutputParamsDictTagged', {
    "@type": typing.Literal["output"],
    "field": str,
})
MrstatsOutputParamsDict = _MrstatsOutputParamsDictNoTag | MrstatsOutputParamsDictTagged


_MrstatsConfigParamsDictNoTag = typing.TypedDict('_MrstatsConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrstatsConfigParamsDictTagged = typing.TypedDict('MrstatsConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrstatsConfigParamsDict = _MrstatsConfigParamsDictNoTag | MrstatsConfigParamsDictTagged


_MrstatsParamsDictNoTag = typing.TypedDict('_MrstatsParamsDictNoTag', {
    "output": typing.NotRequired[list[MrstatsOutputParamsDict] | None],
    "mask": typing.NotRequired[InputPathType | None],
    "ignorezero": bool,
    "allvolumes": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrstatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": InputPathType,
})
MrstatsParamsDictTagged = typing.TypedDict('MrstatsParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrstats"],
    "output": typing.NotRequired[list[MrstatsOutputParamsDict] | None],
    "mask": typing.NotRequired[InputPathType | None],
    "ignorezero": bool,
    "allvolumes": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrstatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": InputPathType,
})
MrstatsParamsDict = _MrstatsParamsDictNoTag | MrstatsParamsDictTagged


def mrstats_output(
    field: str,
) -> MrstatsOutputParamsDictTagged:
    """
    Build parameters.
    
    Args:
        field: output only the field specified. Multiple such options can be\
            supplied if required. Choices are: mean, median, std, std_rv, min, max,\
            count. Useful for use in scripts. Both std options refer to the\
            unbiased (sample) standard deviation. For complex data, min, max and\
            std are calculated separately for real and imaginary parts, std_rv is\
            based on the real valued variance (equals sqrt of sum of variances of\
            imaginary and real parts).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "output",
        "field": field,
    }
    return params


def mrstats_output_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrstatsOutputParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("field", None) is None:
        raise StyxValidationError("`field` must not be None")
    if not isinstance(params["field"], str):
        raise StyxValidationError(f'`field` has the wrong type: Received `{type(params.get("field", None))}` expected `str`')


def mrstats_output_cargs(
    params: MrstatsOutputParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-output")
    cargs.append(params.get("field", None))
    return cargs


def mrstats_config(
    key: str,
    value: str,
) -> MrstatsConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrstats_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrstatsConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrstats_config_cargs(
    params: MrstatsConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrstatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrstatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mrstats_params(
    image: InputPathType,
    output: list[MrstatsOutputParamsDict] | None = None,
    mask: InputPathType | None = None,
    ignorezero: bool = False,
    allvolumes: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrstatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrstatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image: the input image from which statistics will be computed.
        output: output only the field specified. Multiple such options can be\
            supplied if required. Choices are: mean, median, std, std_rv, min, max,\
            count. Useful for use in scripts. Both std options refer to the\
            unbiased (sample) standard deviation. For complex data, min, max and\
            std are calculated separately for real and imaginary parts, std_rv is\
            based on the real valued variance (equals sqrt of sum of variances of\
            imaginary and real parts).
        mask: only perform computation within the specified binary mask image.
        ignorezero: ignore zero values during statistics calculation.
        allvolumes: generate statistics across all image volumes, rather than\
            one set of statistics per image volume.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrstats",
        "ignorezero": ignorezero,
        "allvolumes": allvolumes,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "image": image,
    }
    if output is not None:
        params["output"] = output
    if mask is not None:
        params["mask"] = mask
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrstats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrstatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("output", None) is not None:
        if not isinstance(params["output"], list):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `list[MrstatsOutputParamsDict] | None`')
        for e in params["output"]:
            mrstats_output_validate(e)
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("ignorezero", False) is None:
        raise StyxValidationError("`ignorezero` must not be None")
    if not isinstance(params["ignorezero"], bool):
        raise StyxValidationError(f'`ignorezero` has the wrong type: Received `{type(params.get("ignorezero", False))}` expected `bool`')
    if params.get("allvolumes", False) is None:
        raise StyxValidationError("`allvolumes` must not be None")
    if not isinstance(params["allvolumes"], bool):
        raise StyxValidationError(f'`allvolumes` has the wrong type: Received `{type(params.get("allvolumes", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrstatsConfigParamsDict] | None`')
        for e in params["config"]:
            mrstats_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("image", None) is None:
        raise StyxValidationError("`image` must not be None")
    if not isinstance(params["image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `InputPathType`')


def mrstats_cargs(
    params: MrstatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrstats")
    if params.get("output", None) is not None:
        cargs.extend([a for c in [mrstats_output_cargs(s, execution) for s in params.get("output", None)] for a in c])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("ignorezero", False):
        cargs.append("-ignorezero")
    if params.get("allvolumes", False):
        cargs.append("-allvolumes")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrstats_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("image", None)))
    return cargs


def mrstats_outputs(
    params: MrstatsParamsDict,
    execution: Execution,
) -> MrstatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrstatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def mrstats_execute(
    params: MrstatsParamsDict,
    runner: Runner | None = None,
) -> MrstatsOutputs:
    """
    mrstats
    
    Compute images statistics.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrstatsOutputs`).
    """
    mrstats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRSTATS_METADATA)
    params = execution.params(params)
    cargs = mrstats_cargs(params, execution)
    ret = mrstats_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrstats(
    image: InputPathType,
    output: list[MrstatsOutputParamsDict] | None = None,
    mask: InputPathType | None = None,
    ignorezero: bool = False,
    allvolumes: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrstatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrstatsOutputs:
    """
    mrstats
    
    Compute images statistics.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        image: the input image from which statistics will be computed.
        output: output only the field specified. Multiple such options can be\
            supplied if required. Choices are: mean, median, std, std_rv, min, max,\
            count. Useful for use in scripts. Both std options refer to the\
            unbiased (sample) standard deviation. For complex data, min, max and\
            std are calculated separately for real and imaginary parts, std_rv is\
            based on the real valued variance (equals sqrt of sum of variances of\
            imaginary and real parts).
        mask: only perform computation within the specified binary mask image.
        ignorezero: ignore zero values during statistics calculation.
        allvolumes: generate statistics across all image volumes, rather than\
            one set of statistics per image volume.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrstatsOutputs`).
    """
    params = mrstats_params(
        output=output,
        mask=mask,
        ignorezero=ignorezero,
        allvolumes=allvolumes,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        image=image,
    )
    return mrstats_execute(params, runner)


__all__ = [
    "MRSTATS_METADATA",
    "MrstatsConfigParamsDict",
    "MrstatsConfigParamsDictTagged",
    "MrstatsOutputParamsDict",
    "MrstatsOutputParamsDictTagged",
    "MrstatsOutputs",
    "MrstatsParamsDict",
    "MrstatsParamsDictTagged",
    "mrstats",
    "mrstats_config",
    "mrstats_execute",
    "mrstats_output",
    "mrstats_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DWI2FOD_METADATA = Metadata(
    id="6d16386fcfa6e4701ad68f4333de7173683fda45.boutiques",
    name="dwi2fod",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Dwi2fodFslgradParamsDictNoTag = typing.TypedDict('_Dwi2fodFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Dwi2fodFslgradParamsDictTagged = typing.TypedDict('Dwi2fodFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Dwi2fodFslgradParamsDict = _Dwi2fodFslgradParamsDictNoTag | Dwi2fodFslgradParamsDictTagged


_Dwi2fodVariousStringParamsDictNoTag = typing.TypedDict('_Dwi2fodVariousStringParamsDictNoTag', {
    "obj": str,
})
Dwi2fodVariousStringParamsDictTagged = typing.TypedDict('Dwi2fodVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
Dwi2fodVariousStringParamsDict = _Dwi2fodVariousStringParamsDictNoTag | Dwi2fodVariousStringParamsDictTagged


_Dwi2fodVariousFileParamsDictNoTag = typing.TypedDict('_Dwi2fodVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
Dwi2fodVariousFileParamsDictTagged = typing.TypedDict('Dwi2fodVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
Dwi2fodVariousFileParamsDict = _Dwi2fodVariousFileParamsDictNoTag | Dwi2fodVariousFileParamsDictTagged


_Dwi2fodConfigParamsDictNoTag = typing.TypedDict('_Dwi2fodConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Dwi2fodConfigParamsDictTagged = typing.TypedDict('Dwi2fodConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Dwi2fodConfigParamsDict = _Dwi2fodConfigParamsDictNoTag | Dwi2fodConfigParamsDictTagged


_Dwi2fodResponseOdfParamsDictNoTag = typing.TypedDict('_Dwi2fodResponseOdfParamsDictNoTag', {
    "response": InputPathType,
    "odf": str,
})
Dwi2fodResponseOdfParamsDictTagged = typing.TypedDict('Dwi2fodResponseOdfParamsDictTagged', {
    "@type": typing.Literal["response_odf"],
    "response": InputPathType,
    "odf": str,
})
Dwi2fodResponseOdfParamsDict = _Dwi2fodResponseOdfParamsDictNoTag | Dwi2fodResponseOdfParamsDictTagged


_Dwi2fodParamsDictNoTag = typing.TypedDict('_Dwi2fodParamsDictNoTag', {
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Dwi2fodFslgradParamsDict | None],
    "shells": typing.NotRequired[list[float] | None],
    "directions": typing.NotRequired[InputPathType | None],
    "lmax": typing.NotRequired[list[int] | None],
    "mask": typing.NotRequired[InputPathType | None],
    "filter": typing.NotRequired[InputPathType | None],
    "neg_lambda": typing.NotRequired[float | None],
    "norm_lambda": typing.NotRequired[float | None],
    "threshold": typing.NotRequired[float | None],
    "niter": typing.NotRequired[int | None],
    "norm_lambda_1": typing.NotRequired[float | None],
    "neg_lambda_1": typing.NotRequired[float | None],
    "predicted_signal": typing.NotRequired[str | None],
    "strides": typing.NotRequired[typing.Union[Dwi2fodVariousStringParamsDictTagged, Dwi2fodVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Dwi2fodConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "algorithm": str,
    "dwi": InputPathType,
    "response_odf": list[Dwi2fodResponseOdfParamsDict],
})
Dwi2fodParamsDictTagged = typing.TypedDict('Dwi2fodParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dwi2fod"],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Dwi2fodFslgradParamsDict | None],
    "shells": typing.NotRequired[list[float] | None],
    "directions": typing.NotRequired[InputPathType | None],
    "lmax": typing.NotRequired[list[int] | None],
    "mask": typing.NotRequired[InputPathType | None],
    "filter": typing.NotRequired[InputPathType | None],
    "neg_lambda": typing.NotRequired[float | None],
    "norm_lambda": typing.NotRequired[float | None],
    "threshold": typing.NotRequired[float | None],
    "niter": typing.NotRequired[int | None],
    "norm_lambda_1": typing.NotRequired[float | None],
    "neg_lambda_1": typing.NotRequired[float | None],
    "predicted_signal": typing.NotRequired[str | None],
    "strides": typing.NotRequired[typing.Union[Dwi2fodVariousStringParamsDictTagged, Dwi2fodVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Dwi2fodConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "algorithm": str,
    "dwi": InputPathType,
    "response_odf": list[Dwi2fodResponseOdfParamsDict],
})
Dwi2fodParamsDict = _Dwi2fodParamsDictNoTag | Dwi2fodParamsDictTagged


def dwi2fod_strides_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": dwi2fod_various_string_cargs,
        "VariousFile": dwi2fod_various_file_cargs,
    }.get(t)


def dwi2fod_strides_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": dwi2fod_various_string_validate,
        "VariousFile": dwi2fod_various_file_validate,
    }.get(t)


def dwi2fod_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> Dwi2fodFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dwi2fod_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2fodFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dwi2fod_fslgrad_cargs(
    params: Dwi2fodFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def dwi2fod_various_string(
    obj: str,
) -> Dwi2fodVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def dwi2fod_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2fodVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def dwi2fod_various_string_cargs(
    params: Dwi2fodVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def dwi2fod_various_file(
    obj: InputPathType,
) -> Dwi2fodVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def dwi2fod_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2fodVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def dwi2fod_various_file_cargs(
    params: Dwi2fodVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def dwi2fod_config(
    key: str,
    value: str,
) -> Dwi2fodConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dwi2fod_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2fodConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dwi2fod_config_cargs(
    params: Dwi2fodConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Dwi2fodResponseOdfOutputs(typing.NamedTuple):
    """
    Output object returned when calling `list[Dwi2fodResponseOdfParamsDict](...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    odf: OutputPathType
    """output ODF image"""


def dwi2fod_response_odf(
    response: InputPathType,
    odf: str,
) -> Dwi2fodResponseOdfParamsDictTagged:
    """
    Build parameters.
    
    Args:
        response: input tissue response.
        odf: output ODF image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "response_odf",
        "response": response,
        "odf": odf,
    }
    return params


def dwi2fod_response_odf_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2fodResponseOdfParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("response", None) is None:
        raise StyxValidationError("`response` must not be None")
    if not isinstance(params["response"], (pathlib.Path, str)):
        raise StyxValidationError(f'`response` has the wrong type: Received `{type(params.get("response", None))}` expected `InputPathType`')
    if params.get("odf", None) is None:
        raise StyxValidationError("`odf` must not be None")
    if not isinstance(params["odf"], str):
        raise StyxValidationError(f'`odf` has the wrong type: Received `{type(params.get("odf", None))}` expected `str`')


def dwi2fod_response_odf_cargs(
    params: Dwi2fodResponseOdfParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("response", None)))
    cargs.append(params.get("odf", None))
    return cargs


def dwi2fod_response_odf_outputs(
    params: Dwi2fodResponseOdfParamsDict,
    execution: Execution,
) -> Dwi2fodResponseOdfOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Dwi2fodResponseOdfOutputs(
        root=execution.output_file("."),
        odf=execution.output_file(params.get("odf", None)),
    )
    return ret


class Dwi2fodOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Dwi2fodParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    predicted_signal: OutputPathType | None
    """output the predicted dwi image. """
    response_odf: list[Dwi2fodResponseOdfOutputs]
    """Outputs from `dwi2fod_response_odf_outputs`.This is a list of outputs
    with the same length and order as the inputs."""


def dwi2fod_params(
    algorithm: str,
    dwi: InputPathType,
    response_odf: list[Dwi2fodResponseOdfParamsDict],
    grad: InputPathType | None = None,
    fslgrad: Dwi2fodFslgradParamsDict | None = None,
    shells: list[float] | None = None,
    directions: InputPathType | None = None,
    lmax: list[int] | None = None,
    mask: InputPathType | None = None,
    filter_: InputPathType | None = None,
    neg_lambda: float | None = None,
    norm_lambda: float | None = None,
    threshold: float | None = None,
    niter: int | None = None,
    norm_lambda_1: float | None = None,
    neg_lambda_1: float | None = None,
    predicted_signal: str | None = None,
    strides: typing.Union[Dwi2fodVariousStringParamsDictTagged, Dwi2fodVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Dwi2fodConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Dwi2fodParamsDictTagged:
    """
    Build parameters.
    
    Args:
        algorithm: the algorithm to use for FOD estimation. (options are:\
            csd,msmt_csd).
        dwi: the input diffusion-weighted image.
        response_odf: pairs of input tissue response and output ODF images.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        directions: specify the directions over which to apply the\
            non-negativity constraint (by default, the built-in 300 direction set\
            is used). These should be supplied as a text file containing [ az el ]\
            pairs for the directions.
        lmax: the maximum spherical harmonic order for the output FOD(s).For\
            algorithms with multiple outputs, this should be provided as a\
            comma-separated list of integers, one for each output image; for\
            single-output algorithms, only a single integer should be provided. If\
            omitted, the command will use the lmax of the corresponding response\
            function (i.e based on its number of coefficients), up to a maximum of\
            8.
        mask: only perform computation within the specified binary brain mask\
            image.
        filter_: the linear frequency filtering parameters used for the initial\
            linear spherical deconvolution step (default = [ 1 1 1 0 0 ]). These\
            should be supplied as a text file containing the filtering coefficients\
            for each even harmonic order.
        neg_lambda: the regularisation parameter lambda that controls the\
            strength of the non-negativity constraint (default = 1).
        norm_lambda: the regularisation parameter lambda that controls the\
            strength of the constraint on the norm of the solution (default = 1).
        threshold: the threshold below which the amplitude of the FOD is\
            assumed to be zero, expressed as an absolute amplitude (default = 0).
        niter: the maximum number of iterations to perform for each voxel\
            (default = 50). Use '-niter 0' for a linear unconstrained spherical\
            deconvolution.
        norm_lambda_1: the regularisation parameter lambda that controls the\
            strength of the constraint on the norm of the solution (default =\
            1e-10).
        neg_lambda_1: the regularisation parameter lambda that controls the\
            strength of the non-negativity constraint (default = 1e-10).
        predicted_signal: output the predicted dwi image.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dwi2fod",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "algorithm": algorithm,
        "dwi": dwi,
        "response_odf": response_odf,
    }
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if shells is not None:
        params["shells"] = shells
    if directions is not None:
        params["directions"] = directions
    if lmax is not None:
        params["lmax"] = lmax
    if mask is not None:
        params["mask"] = mask
    if filter_ is not None:
        params["filter"] = filter_
    if neg_lambda is not None:
        params["neg_lambda"] = neg_lambda
    if norm_lambda is not None:
        params["norm_lambda"] = norm_lambda
    if threshold is not None:
        params["threshold"] = threshold
    if niter is not None:
        params["niter"] = niter
    if norm_lambda_1 is not None:
        params["norm_lambda_1"] = norm_lambda_1
    if neg_lambda_1 is not None:
        params["neg_lambda_1"] = neg_lambda_1
    if predicted_signal is not None:
        params["predicted_signal"] = predicted_signal
    if strides is not None:
        params["strides"] = strides
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dwi2fod_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2fodParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dwi2fod_fslgrad_validate(params["fslgrad"])
    if params.get("shells", None) is not None:
        if not isinstance(params["shells"], list):
            raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
        for e in params["shells"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
    if params.get("directions", None) is not None:
        if not isinstance(params["directions"], (pathlib.Path, str)):
            raise StyxValidationError(f'`directions` has the wrong type: Received `{type(params.get("directions", None))}` expected `InputPathType | None`')
    if params.get("lmax", None) is not None:
        if not isinstance(params["lmax"], list):
            raise StyxValidationError(f'`lmax` has the wrong type: Received `{type(params.get("lmax", None))}` expected `list[int] | None`')
        for e in params["lmax"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`lmax` has the wrong type: Received `{type(params.get("lmax", None))}` expected `list[int] | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("filter", None) is not None:
        if not isinstance(params["filter"], (pathlib.Path, str)):
            raise StyxValidationError(f'`filter` has the wrong type: Received `{type(params.get("filter", None))}` expected `InputPathType | None`')
    if params.get("neg_lambda", None) is not None:
        if not isinstance(params["neg_lambda"], (float, int)):
            raise StyxValidationError(f'`neg_lambda` has the wrong type: Received `{type(params.get("neg_lambda", None))}` expected `float | None`')
    if params.get("norm_lambda", None) is not None:
        if not isinstance(params["norm_lambda"], (float, int)):
            raise StyxValidationError(f'`norm_lambda` has the wrong type: Received `{type(params.get("norm_lambda", None))}` expected `float | None`')
    if params.get("threshold", None) is not None:
        if not isinstance(params["threshold"], (float, int)):
            raise StyxValidationError(f'`threshold` has the wrong type: Received `{type(params.get("threshold", None))}` expected `float | None`')
    if params.get("niter", None) is not None:
        if not isinstance(params["niter"], int):
            raise StyxValidationError(f'`niter` has the wrong type: Received `{type(params.get("niter", None))}` expected `int | None`')
    if params.get("norm_lambda_1", None) is not None:
        if not isinstance(params["norm_lambda_1"], (float, int)):
            raise StyxValidationError(f'`norm_lambda_1` has the wrong type: Received `{type(params.get("norm_lambda_1", None))}` expected `float | None`')
    if params.get("neg_lambda_1", None) is not None:
        if not isinstance(params["neg_lambda_1"], (float, int)):
            raise StyxValidationError(f'`neg_lambda_1` has the wrong type: Received `{type(params.get("neg_lambda_1", None))}` expected `float | None`')
    if params.get("predicted_signal", None) is not None:
        if not isinstance(params["predicted_signal"], str):
            raise StyxValidationError(f'`predicted_signal` has the wrong type: Received `{type(params.get("predicted_signal", None))}` expected `str | None`')
    if params.get("strides", None) is not None:
        if not isinstance(params["strides"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["strides"])}\'')
        if "@type" not in params["strides"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["strides"]["@type"] not in ["VariousString", "VariousFile"]:
            raise StyxValidationError("Parameter `strides`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
        dwi2fod_strides_validate_dyn_fn(params["strides"]["@type"])(params["strides"])
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Dwi2fodConfigParamsDict] | None`')
        for e in params["config"]:
            dwi2fod_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("algorithm", None) is None:
        raise StyxValidationError("`algorithm` must not be None")
    if not isinstance(params["algorithm"], str):
        raise StyxValidationError(f'`algorithm` has the wrong type: Received `{type(params.get("algorithm", None))}` expected `str`')
    if params.get("dwi", None) is None:
        raise StyxValidationError("`dwi` must not be None")
    if not isinstance(params["dwi"], (pathlib.Path, str)):
        raise StyxValidationError(f'`dwi` has the wrong type: Received `{type(params.get("dwi", None))}` expected `InputPathType`')
    if params.get("response_odf", None) is None:
        raise StyxValidationError("`response_odf` must not be None")
    if not isinstance(params["response_odf"], list):
        raise StyxValidationError(f'`response_odf` has the wrong type: Received `{type(params.get("response_odf", None))}` expected `list[Dwi2fodResponseOdfParamsDict]`')
    for e in params["response_odf"]:
        dwi2fod_response_odf_validate(e)


def dwi2fod_cargs(
    params: Dwi2fodParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dwi2fod")
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dwi2fod_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("shells", None) is not None:
        cargs.extend([
            "-shells",
            ",".join(map(str, params.get("shells", None)))
        ])
    if params.get("directions", None) is not None:
        cargs.extend([
            "-directions",
            execution.input_file(params.get("directions", None))
        ])
    if params.get("lmax", None) is not None:
        cargs.extend([
            "-lmax",
            ",".join(map(str, params.get("lmax", None)))
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("filter", None) is not None:
        cargs.extend([
            "-filter",
            execution.input_file(params.get("filter", None))
        ])
    if params.get("neg_lambda", None) is not None:
        cargs.extend([
            "-neg_lambda",
            str(params.get("neg_lambda", None))
        ])
    if params.get("norm_lambda", None) is not None:
        cargs.extend([
            "-norm_lambda",
            str(params.get("norm_lambda", None))
        ])
    if params.get("threshold", None) is not None:
        cargs.extend([
            "-threshold",
            str(params.get("threshold", None))
        ])
    if params.get("niter", None) is not None:
        cargs.extend([
            "-niter",
            str(params.get("niter", None))
        ])
    if params.get("norm_lambda_1", None) is not None:
        cargs.extend([
            "-norm_lambda",
            str(params.get("norm_lambda_1", None))
        ])
    if params.get("neg_lambda_1", None) is not None:
        cargs.extend([
            "-neg_lambda",
            str(params.get("neg_lambda_1", None))
        ])
    if params.get("predicted_signal", None) is not None:
        cargs.extend([
            "-predicted_signal",
            params.get("predicted_signal", None)
        ])
    if params.get("strides", None) is not None:
        cargs.extend([
            "-strides",
            *dwi2fod_strides_cargs_dyn_fn(params.get("strides", None)["@type"])(params.get("strides", None), execution)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dwi2fod_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(params.get("algorithm", None))
    cargs.append(execution.input_file(params.get("dwi", None)))
    cargs.extend([a for c in [dwi2fod_response_odf_cargs(s, execution) for s in params.get("response_odf", None)] for a in c])
    return cargs


def dwi2fod_outputs(
    params: Dwi2fodParamsDict,
    execution: Execution,
) -> Dwi2fodOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Dwi2fodOutputs(
        root=execution.output_file("."),
        predicted_signal=execution.output_file(params.get("predicted_signal", None)) if (params.get("predicted_signal") is not None) else None,
        response_odf=[dwi2fod_response_odf_outputs(i, execution) if dwi2fod_response_odf_outputs else None for i in params.get("response_odf")],
    )
    return ret


def dwi2fod_execute(
    params: Dwi2fodParamsDict,
    runner: Runner | None = None,
) -> Dwi2fodOutputs:
    """
    dwi2fod
    
    Estimate fibre orientation distributions from diffusion data using spherical
    deconvolution.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    * If using csd algorithm:
    Tournier, J.-D.; Calamante, F. & Connelly, A. Robust determination of the
    fibre orientation distribution in diffusion MRI: Non-negativity constrained
    super-resolved spherical deconvolution. NeuroImage, 2007, 35, 1459-1472
    
    * If using msmt_csd algorithm:
    Jeurissen, B; Tournier, J-D; Dhollander, T; Connelly, A & Sijbers, J.
    Multi-tissue constrained spherical deconvolution for improved analysis of
    multi-shell diffusion MRI data. NeuroImage, 2014, 103, 411-426
    
    Tournier, J.-D.; Calamante, F., Gadian, D.G. & Connelly, A. Direct
    estimation of the fiber orientation density function from diffusion-weighted
    MRI data using spherical deconvolution. NeuroImage, 2004, 23, 1176-1185.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Dwi2fodOutputs`).
    """
    dwi2fod_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWI2FOD_METADATA)
    params = execution.params(params)
    cargs = dwi2fod_cargs(params, execution)
    ret = dwi2fod_outputs(params, execution)
    execution.run(cargs)
    return ret


def dwi2fod(
    algorithm: str,
    dwi: InputPathType,
    response_odf: list[Dwi2fodResponseOdfParamsDict],
    grad: InputPathType | None = None,
    fslgrad: Dwi2fodFslgradParamsDict | None = None,
    shells: list[float] | None = None,
    directions: InputPathType | None = None,
    lmax: list[int] | None = None,
    mask: InputPathType | None = None,
    filter_: InputPathType | None = None,
    neg_lambda: float | None = None,
    norm_lambda: float | None = None,
    threshold: float | None = None,
    niter: int | None = None,
    norm_lambda_1: float | None = None,
    neg_lambda_1: float | None = None,
    predicted_signal: str | None = None,
    strides: typing.Union[Dwi2fodVariousStringParamsDictTagged, Dwi2fodVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Dwi2fodConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Dwi2fodOutputs:
    """
    dwi2fod
    
    Estimate fibre orientation distributions from diffusion data using spherical
    deconvolution.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    * If using csd algorithm:
    Tournier, J.-D.; Calamante, F. & Connelly, A. Robust determination of the
    fibre orientation distribution in diffusion MRI: Non-negativity constrained
    super-resolved spherical deconvolution. NeuroImage, 2007, 35, 1459-1472
    
    * If using msmt_csd algorithm:
    Jeurissen, B; Tournier, J-D; Dhollander, T; Connelly, A & Sijbers, J.
    Multi-tissue constrained spherical deconvolution for improved analysis of
    multi-shell diffusion MRI data. NeuroImage, 2014, 103, 411-426
    
    Tournier, J.-D.; Calamante, F., Gadian, D.G. & Connelly, A. Direct
    estimation of the fiber orientation density function from diffusion-weighted
    MRI data using spherical deconvolution. NeuroImage, 2004, 23, 1176-1185.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        algorithm: the algorithm to use for FOD estimation. (options are:\
            csd,msmt_csd).
        dwi: the input diffusion-weighted image.
        response_odf: pairs of input tissue response and output ODF images.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        directions: specify the directions over which to apply the\
            non-negativity constraint (by default, the built-in 300 direction set\
            is used). These should be supplied as a text file containing [ az el ]\
            pairs for the directions.
        lmax: the maximum spherical harmonic order for the output FOD(s).For\
            algorithms with multiple outputs, this should be provided as a\
            comma-separated list of integers, one for each output image; for\
            single-output algorithms, only a single integer should be provided. If\
            omitted, the command will use the lmax of the corresponding response\
            function (i.e based on its number of coefficients), up to a maximum of\
            8.
        mask: only perform computation within the specified binary brain mask\
            image.
        filter_: the linear frequency filtering parameters used for the initial\
            linear spherical deconvolution step (default = [ 1 1 1 0 0 ]). These\
            should be supplied as a text file containing the filtering coefficients\
            for each even harmonic order.
        neg_lambda: the regularisation parameter lambda that controls the\
            strength of the non-negativity constraint (default = 1).
        norm_lambda: the regularisation parameter lambda that controls the\
            strength of the constraint on the norm of the solution (default = 1).
        threshold: the threshold below which the amplitude of the FOD is\
            assumed to be zero, expressed as an absolute amplitude (default = 0).
        niter: the maximum number of iterations to perform for each voxel\
            (default = 50). Use '-niter 0' for a linear unconstrained spherical\
            deconvolution.
        norm_lambda_1: the regularisation parameter lambda that controls the\
            strength of the constraint on the norm of the solution (default =\
            1e-10).
        neg_lambda_1: the regularisation parameter lambda that controls the\
            strength of the non-negativity constraint (default = 1e-10).
        predicted_signal: output the predicted dwi image.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Dwi2fodOutputs`).
    """
    params = dwi2fod_params(
        grad=grad,
        fslgrad=fslgrad,
        shells=shells,
        directions=directions,
        lmax=lmax,
        mask=mask,
        filter_=filter_,
        neg_lambda=neg_lambda,
        norm_lambda=norm_lambda,
        threshold=threshold,
        niter=niter,
        norm_lambda_1=norm_lambda_1,
        neg_lambda_1=neg_lambda_1,
        predicted_signal=predicted_signal,
        strides=strides,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        algorithm=algorithm,
        dwi=dwi,
        response_odf=response_odf,
    )
    return dwi2fod_execute(params, runner)


__all__ = [
    "DWI2FOD_METADATA",
    "Dwi2fodConfigParamsDict",
    "Dwi2fodConfigParamsDictTagged",
    "Dwi2fodFslgradParamsDict",
    "Dwi2fodFslgradParamsDictTagged",
    "Dwi2fodOutputs",
    "Dwi2fodParamsDict",
    "Dwi2fodParamsDictTagged",
    "Dwi2fodResponseOdfOutputs",
    "Dwi2fodResponseOdfParamsDict",
    "Dwi2fodResponseOdfParamsDictTagged",
    "Dwi2fodVariousFileParamsDict",
    "Dwi2fodVariousFileParamsDictTagged",
    "Dwi2fodVariousStringParamsDict",
    "Dwi2fodVariousStringParamsDictTagged",
    "dwi2fod",
    "dwi2fod_config",
    "dwi2fod_execute",
    "dwi2fod_fslgrad",
    "dwi2fod_params",
    "dwi2fod_response_odf",
    "dwi2fod_various_file",
    "dwi2fod_various_string",
]
